<?php
/**
 * WooCommerce Box Packer class file.
 *
 * @package WC_Shipping_Royalmail
 */

/**
 * WooCommerce Box Packer
 *
 * @version 2.0.1
 */
class WC_Boxpack {

	/**
	 * List of boxes.
	 *
	 * @var array
	 */
	private $boxes;

	/**
	 * List of items.
	 *
	 * @var array.
	 */
	private $items;

	/**
	 * List of packages.
	 *
	 * @var array.
	 */
	private $packages;

	/**
	 * List of items that cannot be packed.
	 *
	 * @var array.
	 */
	private $cannot_pack;

	/**
	 * __construct function.
	 *
	 * @return void
	 */
	public function __construct() {
		include_once 'class-wc-boxpack-box.php';
		include_once 'class-wc-boxpack-item.php';
	}

	/**
	 * Clear the registered items.
	 *
	 * @return void
	 */
	public function clear_items() {
		$this->items = array();
	}

	/**
	 * Empty the registered boxes.
	 *
	 * @return void
	 */
	public function clear_boxes() {
		$this->boxes = array();
	}

	/**
	 * Add new boxpack item.
	 *
	 * @param mixed $length Length of box.
	 * @param mixed $width Width of box.
	 * @param mixed $height Height of box.
	 * @param mixed $weight Weight of box.
	 * @param mixed $value Value of box.
	 * @param mixed $meta Meta of box.
	 *
	 * @return void
	 */
	public function add_item( $length, $width, $height, $weight, $value = '', $meta = array() ) {
		$this->items[] = new WC_Boxpack_Item( $length, $width, $height, $weight, $value, $meta );
	}

	/**
	 * Add new boxpack box.
	 *
	 * @param mixed $length Length of box.
	 * @param mixed $width Width of box.
	 * @param mixed $height Height of box.
	 * @param mixed $weight Weight of box.
	 *
	 * @return WC_Boxpack_Box
	 */
	public function add_box( $length, $width, $height, $weight = 0 ) {
		$new_box       = new WC_Boxpack_Box( $length, $width, $height, $weight );
		$this->boxes[] = $new_box;
		return $new_box;
	}

	/**
	 * Get the packages.
	 *
	 * @return array
	 */
	public function get_packages() {
		return $this->packages ? $this->packages : array();
	}

	/**
	 * Pack the items.
	 *
	 * @return void
	 *
	 * @throws Exception When items is 0.
	 */
	public function pack() {
		try {
			// We need items.
			if ( count( $this->items ) === 0 ) {
				throw new Exception( 'No items to pack!' );
			}

			// Clear packages.
			$this->packages = array();

			// Order the boxes by volume.
			$this->boxes = $this->order_boxes( $this->boxes );

			if ( ! $this->boxes ) {
				$this->cannot_pack = $this->items;
				$this->items       = array();
			}

			// Keep looping until packed.
			$num_items = count( $this->items );
			while ( $num_items > 0 ) {
				$this->items       = $this->order_items( $this->items );
				$possible_packages = array();
				$best_package      = '';

				// Attempt to pack all items in each box.
				foreach ( $this->boxes as $box ) {
					$possible_packages[] = $box->pack( $this->items );
				}

				// Find the best success rate.
				$best_percent = 0;

				foreach ( $possible_packages as $package ) {
					if ( $package->percent > $best_percent ) {
						$best_percent = $package->percent;
					}
				}

				if ( 0 === $best_percent ) {
					$this->cannot_pack = $this->items;
					$this->items       = array();
				} else {
					// Get smallest box with best_percent.
					$possible_packages = array_reverse( $possible_packages );

					foreach ( $possible_packages as $package ) {
						if ( $package->percent === $best_percent ) {
							$best_package = $package;
							break; // Done packing.
						}
					}

					// Update items array.
					$this->items = $best_package->unpacked;

					// Store package.
					$this->packages[] = $best_package;
				}

				$num_items = count( $this->items );
			}

			// Items we cannot pack (by now) get packaged individually.
			if ( $this->cannot_pack ) {
				foreach ( $this->cannot_pack as $item ) {
					$package           = new stdClass();
					$package->id       = '';
					$package->weight   = $item->get_weight();
					$package->length   = $item->get_length();
					$package->width    = $item->get_width();
					$package->height   = $item->get_height();
					$package->value    = $item->get_value();
					$package->unpacked = true;
					$this->packages[]  = $package;
				}
			}
		} catch ( Exception $e ) {

			// Display a packing error for admins.
			if ( current_user_can( 'manage_options' ) ) {
				echo esc_html( 'Packing error: ' . $e->getMessage() . "\n" );
			}
		}
	}

	/**
	 * Order boxes by weight and volume.
	 *
	 * @param array $sort Boxes to sort.
	 *
	 * @return array
	 */
	private function order_boxes( $sort ) {
		if ( ! empty( $sort ) ) {
			uasort( $sort, array( $this, 'box_sorting' ) );
		}
		return $sort;
	}

	/**
	 * Order items by weight and volume.
	 *
	 * @param array $sort Items to sort.
	 *
	 * @return array
	 */
	private function order_items( $sort ) {
		if ( ! empty( $sort ) ) {
			uasort( $sort, array( $this, 'item_sorting' ) );
		}
		return $sort;
	}

	/**
	 * Order the box based on volume.
	 *
	 * @param array $sort List of box to sort.
	 *
	 * @return array
	 */
	private function order_by_volume( $sort ) {
		if ( ! empty( $sort ) ) {
			uasort( $sort, array( $this, 'volume_based_sorting' ) );
		}
		return $sort;
	}

	/**
	 * Sorting the item using weight and volume.
	 *
	 * @param mixed $a First item.
	 * @param mixed $b Second item.
	 *
	 * @return int
	 */
	public function item_sorting( $a, $b ) {
		if ( $a->get_volume() === $b->get_volume() ) {
			if ( $a->get_weight() === $b->get_weight() ) {
				return 0;
			}
			return ( $a->get_weight() < $b->get_weight() ) ? 1 : -1;
		}
		return ( $a->get_volume() < $b->get_volume() ) ? 1 : -1;
	}

	/**
	 * Sorting the box using weight and volume.
	 *
	 * @param mixed $a First box.
	 * @param mixed $b Second box.
	 *
	 * @return int
	 */
	public function box_sorting( $a, $b ) {
		if ( $a->get_volume() === $b->get_volume() ) {
			if ( $a->get_max_weight() === $b->get_max_weight() ) {
				return 0;
			}
			return ( $a->get_max_weight() < $b->get_max_weight() ) ? 1 : -1;
		}
		return ( $a->get_volume() < $b->get_volume() ) ? 1 : -1;
	}

	/**
	 * Sort base on volume.
	 *
	 * @param mixed $a First package.
	 * @param mixed $b Second package.
	 *
	 * @return int
	 */
	public function volume_based_sorting( $a, $b ) {
		if ( $a->get_volume() === $b->get_volume() ) {
			return 0;
		}
		return ( $a->get_volume() < $b->get_volume() ) ? 1 : -1;
	}
}
