<?php
/**
 * Main Royal Mail class file.
 *
 * @package WC_Shipping_Royalmail
 */

/**
 * Main Royal Mail class
 */
class WC_RoyalMail {
	/**
	 * Plugin's version.
	 *
	 * @since 2.5.0 introduced.
	 *
	 * @var string
	 */
	public $version = WOOCOMMERCE_SHIPPING_ROYALMAIL_VERSION;

	/**
	 * Constructor.
	 */
	public function __construct() {
		add_action( 'admin_init', array( $this, 'maybe_install' ), 5 );
		add_filter( 'plugin_action_links_' . plugin_basename( WOOCOMMERCE_SHIPPING_ROYALMAIL_FILE ), array( $this, 'plugin_action_links' ) );
		add_action( 'woocommerce_shipping_init', array( $this, 'shipping_init' ) );
		add_filter( 'woocommerce_shipping_methods', array( $this, 'shipping_methods' ) );
		add_action( 'admin_notices', array( $this, 'environment_check' ) );
		add_action( 'before_woocommerce_init', array( $this, 'declare_hpos_compatibility' ) );

		include_once WOOCOMMERCE_SHIPPING_ROYALMAIL_ABSPATH . 'includes/class-wc-shipping-royalmail-admin.php';
		if ( is_admin() ) {
			new WC_Shipping_Royalmail_Admin();
		}
	}

	/**
	 * Check environment.
	 *
	 * Hooked into `admin_notices` so it'll render admin notice if there's
	 * a failed check.
	 *
	 * @return void
	 */
	public function environment_check() {

		if ( 'GBP' !== get_woocommerce_currency() ) {
			echo '<div class="error">
				<p>' .
				// translators: %s is a link to currency settings.
				wp_kses_post( sprintf( __( 'Royal Mail requires that the <a href="%s">currency</a> is set to Pound sterling.', 'woocommerce-shipping-royalmail' ), esc_url( admin_url( 'admin.php?page=wc-settings&tab=general' ) ) ) ) .
				'</p>
			</div>';
		}

		if ( 'GB' !== WC()->countries->get_base_country() ) {
			echo '<div class="error">
				<p>' .
				// translators: %s is a link to base country/region settings.
				wp_kses_post( sprintf( __( 'Royal Mail requires that the <a href="%s">base country/region</a> is set to United Kingdom.', 'woocommerce-shipping-royalmail' ), esc_url( admin_url( 'admin.php?page=wc-settings&tab=general' ) ) ) ) .
				'</p>
			</div>';
		}
	}

	/**
	 * Add plugin action links to the plugins page.
	 *
	 * @param array $links Links.
	 *
	 * @return array Links.
	 */
	public function plugin_action_links( $links ) {
		$plugin_links = array(
			'<a href="https://woocommerce.com/my-account/create-a-ticket?broken=primary&select=182719">' . __( 'Support', 'woocommerce-shipping-royalmail' ) . '</a>',
			'<a href="https://www.woocommerce.com/products/royal-mail">' . __( 'Docs', 'woocommerce-shipping-royalmail' ) . '</a>',
		);
		return array_merge( $plugin_links, $links );
	}

	/**
	 * Load our shipping class.
	 */
	public function shipping_init() {
		require_once WOOCOMMERCE_SHIPPING_ROYALMAIL_ABSPATH . 'includes/class-wc-royalmail-privacy.php';
		include_once WOOCOMMERCE_SHIPPING_ROYALMAIL_ABSPATH . 'includes/class-wc-shipping-royalmail.php';
	}

	/**
	 * Add our shipping method to woocommerce.
	 *
	 * @param array $methods Shipping methods.
	 *
	 * @return array Shipping methods.
	 */
	public function shipping_methods( $methods ) {
		$methods['royal_mail'] = 'WC_Shipping_Royalmail';
		return $methods;
	}

	/**
	 * Checks the plugin version.
	 *
	 * @since 2.5.0
	 * @version 2.5.0
	 *
	 * @return bool
	 */
	public function maybe_install() {
		// Only need to do this for versions less than 2.5.0 to migrate
		// settings to shipping zone instance.
		$doing_ajax = defined( 'DOING_AJAX' ) && DOING_AJAX;
		if ( ! $doing_ajax
			&& ! defined( 'IFRAME_REQUEST' )
			&& version_compare( get_option( 'wc_royal_mail_version' ), '2.5.0', '<' ) ) {

			$this->install();

		}

		return true;
	}

	/**
	 * Update/migration script.
	 *
	 * @since 2.5.0
	 * @version 2.5.0
	 */
	public function install() {
		// Get all saved settings and cache it.
		$royal_mail_settings = get_option( 'woocommerce_royal_mail_settings', false );

		// If settings exists.
		if ( $royal_mail_settings ) {
			global $wpdb;

			// Unset un-needed settings.
			unset( $royal_mail_settings['enabled'] );
			unset( $royal_mail_settings['availability'] );
			unset( $royal_mail_settings['countries'] );

			// First add it to the "rest of the world" zone when no Royal Mail
			// instance.
			if ( ! $this->is_zone_has_royal_mail( 0 ) ) {
				$wpdb->query( $wpdb->prepare( "INSERT INTO {$wpdb->prefix}woocommerce_shipping_zone_methods ( zone_id, method_id, method_order, is_enabled ) VALUES ( %d, %s, %d, %d )", 0, 'royal_mail', 1, 1 ) );
				// Add settings to the newly created instance to options table.
				$instance = $wpdb->insert_id;
				add_option( 'woocommerce_royal_mail_' . $instance . '_settings', $royal_mail_settings );
			}
		}
		update_option( 'wc_royal_mail_version', $this->version );
	}

	/**
	 * Helper method to check whether given zone_id has royal_mail method instance.
	 *
	 * @since 2.5.0
	 * @version 2.5.0
	 *
	 * @param int $zone_id Zone ID.
	 *
	 * @return bool True if given zone_id has royal_mail method instance.
	 */
	public function is_zone_has_royal_mail( $zone_id ) {
		global $wpdb;

		return (int) $wpdb->get_var( $wpdb->prepare( "SELECT COUNT(instance_id) FROM {$wpdb->prefix}woocommerce_shipping_zone_methods WHERE method_id = 'royal_mail' AND zone_id = %d", $zone_id ) ) > 0;
	}

	/**
	 * Declaring HPOS compatibility.
	 */
	public function declare_hpos_compatibility() {
		if ( class_exists( '\Automattic\WooCommerce\Utilities\FeaturesUtil' ) ) {
			\Automattic\WooCommerce\Utilities\FeaturesUtil::declare_compatibility( 'custom_order_tables', 'woocommerce-shipping-royalmail/woocommerce-shipping-royalmail.php', true );
		}
	}
}
