<?php
/**
 * Shipping method class.
 *
 * @package WC_Shipping_Royalmail
 */

/**
 * WC_Shipping_Royalmail class.
 *
 * @extends WC_Shipping_Method
 */
class WC_Shipping_Royalmail extends WC_Shipping_Method {
	/**
	 * Whether debug setting is enabled or not.
	 *
	 * @var bool.
	 */
	private $debug;

	/**
	 * Packing method that is being used.
	 *
	 * @var String.
	 */
	protected $packing_method = '';

	/**
	 * Which rate types are being used.
	 *
	 * @var String.
	 */
	protected $rate_type = '';

	/**
	 * How the rates being offered.
	 *
	 * @var String.
	 */
	protected $offer_rates = '';

	/**
	 * Whether use compensation or not.
	 *
	 * @var String.
	 */
	protected $compensation_optional = '';

	/**
	 * List of custom services.
	 *
	 * @var Array.
	 */
	public $custom_services = array();

	/**
	 * List of boxes.
	 *
	 * @var Array.
	 */
	protected $boxes = array();

	/**
	 * Pre-defined services.
	 *
	 * @var array
	 */
	private $services = array(
		'first-class'                  => 'Royal Mail 1st Class',
		'first-class-signed'           => 'Royal Mail Signed For&reg; 1st Class',
		'second-class'                 => 'Royal Mail 2nd Class',
		'second-class-signed'          => 'Royal Mail Signed For&reg; 2nd Class',

		'special-delivery-9am'         => 'Royal Mail Special Delivery Guaranteed by 9am&reg;',
		'special-delivery-1pm'         => 'Royal Mail Special Delivery Guaranteed by 1pm&reg;',

		'tracked-24'                   => 'Royal Mail Tracked 24',
		'tracked-24-signed'            => 'Royal Mail Tracked 24 with Signature',
		'tracked-24-age-verification'  => 'Royal Mail Tracked 24 with Age Verification',
		'tracked-48'                   => 'Royal Mail Tracked 48',
		'tracked-48-signed'            => 'Royal Mail Tracked 48 with Signature',
		'tracked-48-age-verification'  => 'Royal Mail Tracked 48 with Age Verification',

		'parcelforce-express-9'        => 'Parcelforce Worldwide Express 9',
		'parcelforce-express-10'       => 'Parcelforce Worldwide Express 10',
		'parcelforce-express-am'       => 'Parcelforce Worldwide Express AM',
		'parcelforce-express-24'       => 'Parcelforce Worldwide Express 24',
		'parcelforce-express-48'       => 'Parcelforce Worldwide Express 48',
		'parcelforce-express-48-large' => 'Parcelforce Worldwide Express 48 Large',

		'international-standard'       => 'Royal Mail International Standard',
		'international-tracked-signed' => 'Royal Mail International Tracked &amp; Signed',
		'international-tracked'        => 'Royal Mail International Tracked',
		'international-signed'         => 'Royal Mail International Signed',
		'international-economy'        => 'Royal Mail International Economy',

		'parcelforce-irelandexpress'   => 'Parcelforce Worldwide Ireland Express',
		'parcelforce-europriority'     => 'Parcelforce Worldwide Euro Priority',
		'parcelforce-globaleconomy'    => 'Parcelforce Worldwide Global Economy',
		'parcelforce-globalexpress'    => 'Parcelforce Worldwide Global Express',
		'parcelforce-globalpriority'   => 'Parcelforce Worldwide Global Priority',
		'parcelforce-globalvalue'      => 'Parcelforce Worldwide Global Value',
	);

	/**
	 * Tax exemption lookup table.
	 *
	 * @var array
	 */
	private $is_taxed = array(
		'first-class'                  => false,
		'first-class-signed'           => false,
		'second-class'                 => false,
		'second-class-signed'          => false,

		'special-delivery-9am'         => true,
		'special-delivery-1pm'         => true,

		'tracked-24'                   => true,
		'tracked-24-signed'            => true,
		'tracked-24-age-verification'  => true,
		'tracked-48'                   => true,
		'tracked-48-signed'            => true,
		'tracked-48-age-verification'  => true,

		'parcelforce-express-9'        => true,
		'parcelforce-express-10'       => true,
		'parcelforce-express-am'       => true,
		'parcelforce-express-24'       => true,
		'parcelforce-express-48'       => true,
		'parcelforce-express-48-large' => true,

		'international-standard'       => false,
		'international-tracked-signed' => false,
		'international-tracked'        => false,
		'international-signed'         => false,
		'international-economy'        => false,

		'parcelforce-irelandexpress'   => true,
		'parcelforce-europriority'     => true,
		'parcelforce-globaleconomy'    => true,
		'parcelforce-globalexpress'    => true,
		'parcelforce-globalpriority'   => true,
		'parcelforce-globalvalue'      => true,
	);

	/**
	 * List of services that are only available at certain post offices.
	 *
	 * @var array
	 */
	private $limited_availability = array(
		'first-class'                  => false,
		'first-class-signed'           => false,
		'second-class'                 => false,
		'second-class-signed'          => false,

		'special-delivery-9am'         => false,
		'special-delivery-1pm'         => false,

		'tracked-24'                   => false,
		'tracked-24-signed'            => false,
		'tracked-24-age-verification'  => false,
		'tracked-48'                   => false,
		'tracked-48-signed'            => false,
		'tracked-48-age-verification'  => false,

		'parcelforce-express-9'        => false,
		'parcelforce-express-10'       => false,
		'parcelforce-express-am'       => false,
		'parcelforce-express-24'       => false,
		'parcelforce-express-48'       => false,
		'parcelforce-express-48-large' => true,

		'international-standard'       => false,
		'international-tracked-signed' => false,
		'international-tracked'        => false,
		'international-signed'         => false,
		'international-economy'        => false,

		'parcelforce-irelandexpress'   => false,
		'parcelforce-europriority'     => false,
		'parcelforce-globaleconomy'    => false,
		'parcelforce-globalexpress'    => false,
		'parcelforce-globalpriority'   => false,
		'parcelforce-globalvalue'      => false,
	);

	/**
	 * Service tips.
	 *
	 * @var array
	 */
	public $service_tips;

	/**
	 * Constructor.
	 *
	 * @param int $instance_id Instance ID.
	 *
	 * @return void
	 */
	public function __construct( $instance_id = 0 ) {
		$this->id                 = 'royal_mail';
		$this->instance_id        = absint( $instance_id );
		$this->method_title       = __( 'Royal Mail', 'woocommerce-shipping-royalmail' );
		$this->method_description = __( 'Offer Royal Mail shipping rates automatically to your customers. Prices according to <a href="https://www.royalmail.com/sites/royalmail.com/files/2024-03/our-prices-april-2024-v1-ta.pdf">the 2024 price guide</a>.', 'woocommerce-shipping-royalmail' );
		$this->supports           = array(
			'shipping-zones',
			'instance-settings',
			'settings',
		);

		$this->service_tips = array(
			'parcelforce-express-48-large' => esc_html__( 'This service is only available at select post offices. Please contact your local branch to inquire about its specific availability.', 'woocommerce-shipping-royalmail' ),
		);
		$this->init();
	}

	/**
	 * Checks whether this shipping method is available or not.
	 *
	 * @param array $package Package to ship.
	 * @return bool
	 */
	public function is_available( $package ) {
		if ( empty( $package['destination']['country'] ) ) {
			return false;
		}

		/**
		 * Filter to allow third party disable/enable the shipping method availability.
		 *
		 * @param boolean Flag for the availability.
		 * @param array Cart package.
		 *
		 * @since 2.5.0
		 */
		return apply_filters( 'woocommerce_shipping_' . $this->id . '_is_available', true, $package );
	}

	/**
	 * Initialize settings.
	 *
	 * @version 2.5.0
	 * @since 2.5.0
	 */
	private function set_settings() {
		// Define user set variables.
		$this->title                 = $this->get_option( 'title', $this->method_title );
		$this->packing_method        = $this->get_option( 'packing_method', 'per_item' );
		$this->rate_type             = $this->get_option( 'rate_type', 'regular' );
		$this->offer_rates           = $this->get_option( 'offer_rates', 'all' );
		$this->compensation_optional = $this->get_option( 'compensation_optional', 'no' );
		$this->debug                 = 'yes' === $this->get_option( 'debug_mode' );
		$this->custom_services       = $this->get_option( 'services', array() );
		$this->boxes                 = $this->get_option( 'boxes', array() );
		$this->tax_status            = $this->get_option( 'tax_status' );
	}

	/**
	 * Init form fields and set properties from saved settings.
	 *
	 * @return void
	 */
	private function init() {
		// Load the settings.
		$this->init_form_fields();
		$this->set_settings();

		add_action( 'woocommerce_update_options_shipping_' . $this->id, array( $this, 'process_admin_options' ) );

		if ( ! defined( 'WC_ROYALMAIL_DEBUG' ) ) {
			define( 'WC_ROYALMAIL_DEBUG', $this->debug );
		}
	}

	/**
	 * Process settings on save
	 *
	 * @return void
	 */
	public function process_admin_options() {
		parent::process_admin_options();

		$this->set_settings();
	}

	/**
	 * Render services matrix input in instance setting page.
	 *
	 * @return string HTML of matrix input
	 */
	public function generate_services_html() {
		ob_start();
		?>
		<tr valign="top" id="wc_royal_mail_service_options">
			<th scope="row" class="titledesc">
				<?php esc_html_e( 'Services', 'woocommerce-shipping-royalmail' ); ?>
				<p style="margin-top:30px;">
					<span style="margin-right:8px; background-color:#ffffb0; width:15px; height:15px; display:block; float:left; position:relative; top:3px;">&nbsp;</span>
					<strong style="display:block; overflow:hidden;"><?php esc_html_e( 'Only available at select post offices', 'woocommerce-shipping-royalmail' ); ?></strong><br />
					<span style="display:block;clear:both;"></span>
					<span style="margin-right:8px; background-color:#ffffff; width:15px; height:15px; display:block; float:left; position:relative; top:3px;">&nbsp;</span>
					<strong style="display:block; overflow:hidden;"><?php esc_html_e( 'Available at all post offices', 'woocommerce-shipping-royalmail' ); ?></strong>
					<span style="display:block;clear:both;"></span>
				</p>
			</th>
			<td class="forminp" rowspan="2">
				<table class="royal_mail_services widefat">
					<thead>
						<th class="sort">&nbsp;</th>
						<th width="1%">&nbsp;</th>
						<th><?php esc_html_e( 'Name', 'woocommerce-shipping-royalmail' ); ?></th>
						<th><?php esc_html_e( 'Enabled', 'woocommerce-shipping-royalmail' ); ?></th>
						<th>
						<?php
						/* translators: currency symbol */
						printf( esc_html__( 'Price Adjustment (%s)', 'woocommerce-shipping-royalmail' ), esc_html( get_woocommerce_currency_symbol() ) );
						?>
						</th>
						<th><?php esc_html_e( 'Price Adjustment (%)', 'woocommerce-shipping-royalmail' ); ?></th>
					</thead>
					<tbody>
						<?php
						$sort             = 0;
						$ordered_services = array();

						foreach ( $this->services as $code => $name ) {

							if ( isset( $this->custom_services[ $code ]['order'] ) ) {
								$sort = $this->custom_services[ $code ]['order'];
							}

							while ( isset( $ordered_services[ $sort ] ) ) {
								++$sort;
							}

							$ordered_services[ $sort ] = array( $code, $name );

							++$sort;
						}

						ksort( $ordered_services );

						foreach ( $ordered_services as $value ) {
							$code = $value[0];
							$name = $value[1];

							$row_color = true === $this->limited_availability[ $code ] ? '#ffffb0' : '#ffffff';
							$tip       = ! empty( $this->service_tips[ $code ] ) ? $this->service_tips[ $code ] : $code;
							?>
							<tr style="background-color:<?php echo esc_attr( $row_color ); ?>">
								<td class="sort"><input type="hidden" class="order" name="<?php echo esc_attr( "royal_mail_service[$code][order]" ); ?>" value="<?php echo esc_attr( isset( $this->custom_services[ $code ]['order'] ) ? $this->custom_services[ $code ]['order'] : '' ); ?>" /></td>
								<td width="1%"><strong><?php echo '<img class="help_tip" data-tip="' . esc_attr( $tip ) . '" src="' . esc_url( WC()->plugin_url() ) . '/assets/images/help.png" height="16" width="16" />'; ?></strong></td>
								<td><input type="text" name="<?php echo esc_attr( "royal_mail_service[$code][name]" ); ?>" placeholder="<?php echo esc_attr( $name ); ?>" value="<?php echo esc_attr( isset( $this->custom_services[ $code ]['name'] ) ? $this->custom_services[ $code ]['name'] : '' ); ?>" size="50" /></td>
								<td width="10%"><input type="checkbox" name="<?php echo esc_attr( "royal_mail_service[$code][enabled]" ); ?>" <?php checked( ! empty( $this->custom_services[ $code ]['enabled'] ), true ); ?> /></td>
								<td><input type="text" name="<?php echo esc_attr( "royal_mail_service[$code][adjustment]" ); ?>" placeholder="N/A" value="<?php echo esc_attr( isset( $this->custom_services[ $code ]['adjustment'] ) ? $this->custom_services[ $code ]['adjustment'] : '' ); ?>" size="4" /></td>
								<td><input type="text" name="<?php echo esc_attr( "royal_mail_service[$code][adjustment_percent]" ); ?>" placeholder="N/A" value="<?php echo esc_attr( isset( $this->custom_services[ $code ]['adjustment_percent'] ) ? $this->custom_services[ $code ]['adjustment_percent'] : '' ); ?>" size="4" /></td>
							</tr>
							<?php
						}
						?>
					</tbody>
				</table>
			</td>
		</tr>
		<?php
		return ob_get_clean();
	}

	/**
	 * Render boxes matrix input.
	 *
	 * @return string HTML boxes matrix input
	 */
	public function generate_box_packing_html() {
		ob_start();
		?>
		<tr valign="top" id="wc_royal_mail_packing_options">
			<th scope="row" class="titledesc"><?php esc_html_e( 'International Parcel Sizes', 'woocommerce-shipping-royalmail' ); ?></th>
			<td class="forminp">
				<style type="text/css">
					.royal_mail_boxes td, .royal_mail_services td {
						vertical-align: middle;
						padding: 4px 7px;
					}
					.royal_mail_boxes th, .royal_mail_services th {
						padding: 9px 7px;
					}
					.royal_mail_boxes td input {
						margin-right: 4px;
					}
					.royal_mail_boxes .check-column {
						vertical-align: middle;
						text-align: left;
						padding: 0 7px;
					}
					.royal_mail_services th.sort {
						width: 16px;
					}
					.royal_mail_services td.sort {
						cursor: move;
						width: 16px;
						padding: 0 16px;
						cursor: move;
						background: url(data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAgAAAAICAYAAADED76LAAAAHUlEQVQYV2O8f//+fwY8gJGgAny6QXKETRgEVgAAXxAVsa5Xr3QAAAAASUVORK5CYII=) no-repeat center;
					}
				</style>
				<table class="royal_mail_boxes widefat">
					<thead>
						<tr>
							<th class="check-column"><input type="checkbox" /></th>
							<th><?php esc_html_e( 'Name', 'woocommerce-shipping-royalmail' ); ?></th>
							<th><?php esc_html_e( 'Length', 'woocommerce-shipping-royalmail' ); ?></th>
							<th><?php esc_html_e( 'Width', 'woocommerce-shipping-royalmail' ); ?></th>
							<th><?php esc_html_e( 'Height', 'woocommerce-shipping-royalmail' ); ?></th>
							<th><?php esc_html_e( 'Weight of Box', 'woocommerce-shipping-royalmail' ); ?></th>
						</tr>
					</thead>
					<tfoot>
						<tr>
							<th colspan="2">
								<a href="#" class="button plus insert"><?php esc_html_e( 'Add Box', 'woocommerce-shipping-royalmail' ); ?></a>
								<a href="#" class="button minus remove"><?php esc_html_e( 'Remove selected box(es)', 'woocommerce-shipping-royalmail' ); ?></a>
							</th>
							<th colspan="3">
								<small class="description"><?php esc_html_e( 'When calculating rates for international mail, items will be packed into these boxes depending on item dimensions and volume. The boxes will then be quoted accordingly.', 'woocommerce-shipping-royalmail' ); ?></small>
								<br/><br/>
								<small class="description"><?php esc_html_e( 'The parcels length, width and depth combined must not be no greater than 900mm. The greatest single dimension must not exceed 600mm', 'woocommerce-shipping-royalmail' ); ?></small>
							</th>
						</tr>
					</tfoot>
					<tbody id="rates">
						<?php
						if ( $this->boxes ) {
							foreach ( $this->boxes as $key => $box ) {
								?>
								<tr>
									<td class="check-column"><input type="checkbox" /></td>
									<td><input type="text" size="10" name="<?php echo esc_attr( "boxes_name[$key]" ); ?>" value="<?php echo isset( $box['name'] ) ? esc_attr( $box['name'] ) : ''; ?>" /></td>
									<td><input type="text" size="5" name="<?php echo esc_attr( "boxes_inner_length[$key]" ); ?>" value="<?php echo esc_attr( $box['inner_length'] ); ?>" />mm</td>
									<td><input type="text" size="5" name="<?php echo esc_attr( "boxes_inner_width[$key]" ); ?>" value="<?php echo esc_attr( $box['inner_width'] ); ?>" />mm</td>
									<td><input type="text" size="5" name="<?php echo esc_attr( "boxes_inner_height[$key]" ); ?>" value="<?php echo esc_attr( $box['inner_height'] ); ?>" />mm</td>
									<td><input type="text" size="5" name="<?php echo esc_attr( "boxes_box_weight[$key]" ); ?>" value="<?php echo esc_attr( $box['box_weight'] ); ?>" />g</td>
								</tr>
								<?php
							}
						}
						?>
					</tbody>
				</table>
				<script type="text/javascript">

					jQuery().ready(function(){

						jQuery('.royal_mail_boxes .insert').click( function() {
							var $tbody = jQuery('.royal_mail_boxes').find('tbody');
							var size = $tbody.find('tr').length;
							var code = '<tr class="new">\
									<td class="check-column"><input type="checkbox" /></td>\
									<td><input type="text" size="10" name="boxes_name[' + size + ']" /></td>\
									<td><input type="text" size="5" name="boxes_inner_length[' + size + ']" />mm</td>\
									<td><input type="text" size="5" name="boxes_inner_width[' + size + ']" />mm</td>\
									<td><input type="text" size="5" name="boxes_inner_height[' + size + ']" />mm</td>\
									<td><input type="text" size="5" name="boxes_box_weight[' + size + ']" />g</td>\
								</tr>';

							$tbody.append( code );

							return false;
						} );

						jQuery('.royal_mail_boxes .remove').click(function() {
							var $tbody = jQuery('.royal_mail_boxes').find('tbody');

							$tbody.find('.check-column input:checked').each(function() {
								jQuery(this).closest('tr').hide().find('input').val('');
							});

							return false;
						});

						// Ordering
						jQuery('.royal_mail_services tbody').sortable({
							items:'tr',
							cursor:'move',
							axis:'y',
							handle: '.sort',
							scrollSensitivity:40,
							forcePlaceholderSize: true,
							helper: 'clone',
							opacity: 0.65,
							placeholder: 'wc-metabox-sortable-placeholder',
							start:function(event,ui){
								ui.item.css('background-color','#f6f6f6');
							},
							stop:function(event,ui){
								ui.item.removeAttr('style');
								royal_mail_services_row_indexes();
							}
						});

						function royal_mail_services_row_indexes() {
							jQuery('.royal_mail_services tbody tr').each(function(index, el){
								jQuery('input.order', el).val( parseInt( jQuery(el).index('.royal_mail_services tr') ) );
							});
						};

					});

				</script>
			</td>
		</tr>
		<?php
		return ob_get_clean();
	}

	/**
	 * Validate box packing fields.
	 *
	 * @param mixed $key Key.
	 * @return array
	 */
	public function validate_box_packing_field( $key ) {
		// No need to verify. It has been verified on `WC_Settings_Shipping::instance_settings_screen()`.
		// phpcs:disable WordPress.Security.NonceVerification.Missing
		$boxes_name         = isset( $_POST['boxes_name'] ) ? wc_clean( wp_unslash( $_POST['boxes_name'] ) ) : array();
		$boxes_inner_length = isset( $_POST['boxes_inner_length'] ) ? wc_clean( wp_unslash( $_POST['boxes_inner_length'] ) ) : array();
		$boxes_inner_width  = isset( $_POST['boxes_inner_width'] ) ? wc_clean( wp_unslash( $_POST['boxes_inner_width'] ) ) : array();
		$boxes_inner_height = isset( $_POST['boxes_inner_height'] ) ? wc_clean( wp_unslash( $_POST['boxes_inner_height'] ) ) : array();
		$boxes_box_weight   = isset( $_POST['boxes_box_weight'] ) ? wc_clean( wp_unslash( $_POST['boxes_box_weight'] ) ) : array();
		// phpcs:enable WordPress.Security.NonceVerification.Missing

		$boxes       = array();
		$box_counter = is_array( $boxes_inner_length ) ? count( $boxes_inner_length ) : 0;

		for ( $i = 0; $i < $box_counter; $i++ ) {

			if (
				! empty( $boxes_name[ $i ] ) &&
				! empty( $boxes_inner_length[ $i ] ) &&
				! empty( $boxes_inner_width[ $i ] ) &&
				! empty( $boxes_inner_height[ $i ] )
			) {
				$boxes[] = array(
					'name'         => $boxes_name[ $i ],
					'inner_length' => floatval( $boxes_inner_length[ $i ] ),
					'inner_width'  => floatval( $boxes_inner_width[ $i ] ),
					'inner_height' => floatval( $boxes_inner_height[ $i ] ),
					'box_weight'   => floatval( $boxes_box_weight[ $i ] ),
				);
			}
		}

		return $boxes;
	}

	/**
	 * Validate services fields.
	 *
	 * @param mixed $key Key.
	 * @return array
	 */
	public function validate_services_field( $key ) {
		// No need to verify. It has been verified on `WC_Settings_Shipping::instance_settings_screen()`.
		// phpcs:disable WordPress.Security.NonceVerification.Missing
		$posted_services = isset( $_POST['royal_mail_service'] ) ? wc_clean( wp_unslash( $_POST['royal_mail_service'] ) ) : array();
		$services        = array();
		// phpcs:enable WordPress.Security.NonceVerification.Missing

		foreach ( $posted_services as $code => $settings ) {
			$services[ $code ] = array(
				'name'               => isset( $settings['name'] ) ? $settings['name'] : '',
				'order'              => isset( $settings['order'] ) ? $settings['order'] : '',
				'enabled'            => isset( $settings['enabled'] ) ? true : false,
				'adjustment'         => isset( $settings['adjustment'] ) ? floatval( $settings['adjustment'] ) : '',
				'adjustment_percent' => isset( $settings['adjustment_percent'] ) ? floatval( str_replace( '%', '', $settings['adjustment_percent'] ) ) : '',
			);
		}

		return $services;
	}

	/**
	 * Init form fields.
	 *
	 * @return void
	 */
	public function init_form_fields() {
		$this->instance_form_fields = array(
			'title'                     => array(
				'title'       => __( 'Method Title', 'woocommerce-shipping-royalmail' ),
				'type'        => 'text',
				'description' => '',
				'default'     => __( 'Royal Mail', 'woocommerce-shipping-royalmail' ),
			),
			'rates'                     => array(
				'title'       => __( 'Rates and Services', 'woocommerce-shipping-royalmail' ),
				'type'        => 'title',
				'description' => '',
			),
			'tax_status'                => array(
				'title'       => __( 'Tax Status', 'woocommerce-shipping-usps' ),
				'type'        => 'select',
				'description' => '',
				'default'     => 'taxable',
				'options'     => array(
					'taxable' => __( 'Taxable', 'woocommerce-shipping-usps' ),
					'none'    => __( 'None', 'woocommerce-shipping-usps' ),
				),
			),
			'packing_method'            => array(
				'title'   => __( 'Parcel Packing Method', 'woocommerce-shipping-royalmail' ),
				'type'    => 'select',
				'default' => '',
				'class'   => 'packing_method',
				'options' => array(
					'per_item'    => __( 'Default: Pack items individually', 'woocommerce-shipping-royalmail' ),
					'box_packing' => __( 'Recommended: Pack items into boxes together', 'woocommerce-shipping-royalmail' ),
				),
			),
			'boxes'                     => array(
				'type' => 'box_packing',
			),
			'rate_type'                 => array(
				'title'       => __( 'Rate Type', 'woocommerce-shipping-royalmail' ),
				'type'        => 'select',
				'description' => '',
				'default'     => 'regular',
				'options'     => array(
					'regular' => __( 'Regular prices', 'woocommerce-shipping-royalmail' ),
					'online'  => __( 'Online prices', 'woocommerce-shipping-royalmail' ),
				),
			),
			'offer_rates'               => array(
				'title'       => __( 'Offer Rates', 'woocommerce-shipping-royalmail' ),
				'type'        => 'select',
				'description' => '',
				'default'     => 'all',
				'options'     => array(
					'all'      => __( 'Offer the customer all returned rates', 'woocommerce-shipping-royalmail' ),
					'cheapest' => __( 'Offer the customer the cheapest rate only, anonymously', 'woocommerce-shipping-royalmail' ),
				),
			),
			'compensation_optional'     => array(
				'title'       => __( 'Ignore Maximum Compensation', 'woocommerce-shipping-royalmail' ),
				'type'        => 'checkbox',
				'description' => __( 'Enabling this will return rates where order amount is greater than what will be compensated.', 'woocommerce-shipping-royalmail' ),
				'default'     => 'no',
			),
			'ignore_max_total_cover'    => array(
				'title'       => __( 'Ignore Maximum Total Cover for Parcelforce', 'woocommerce-shipping-royalmail' ),
				'type'        => 'checkbox',
				'description' => __( 'Enabling this will return Parcelforce rates even when order amount is greater than maximum total cover.', 'woocommerce-shipping-royalmail' ),
				'default'     => 'no',
			),
			'enable_addit_compensation' => array(
				'title'       => __( 'Enable Additional Compensation', 'woocommerce-shipping-royalmail' ),
				'type'        => 'checkbox',
				'description' => __( 'Enabling this will add an additional fee for increasing compensation value up to 250 Poundsterling for certain services.', 'woocommerce-shipping-royalmail' ),
				'default'     => 'no',
			),
			'services'                  => array(
				'type' => 'services',
			),
		);

		$this->form_fields = array(
			'debug_mode' => array(
				'title'       => __( 'Debug Mode', 'wc_australia_post' ),
				'label'       => __( 'Enable debug mode', 'wc_australia_post' ),
				'type'        => 'checkbox',
				'default'     => 'yes',
				'desc_tip'    => true,
				'description' => __( 'Enable debug mode to show debugging information on your cart/checkout.', 'wc_australia_post' ),
			),
		);
	}

	/**
	 * Calculate shipping cost.
	 *
	 * @param mixed $package Package to ship.
	 * @return void
	 */
	public function calculate_shipping( $package = array() ) {
		include_once 'class-wc-shipping-royalmail-rates.php';

		$rates     = array();
		$rates_api = new WC_Shipping_Royalmail_Rates( $package, $this->packing_method, $this->boxes, $this->instance_id, $this->rate_type );
		$quotes    = $rates_api->get_quotes();

		if ( $quotes ) {

			foreach ( $quotes as $rate_code => $cost ) {

				$rate_id       = $this->id . ':' . $rate_code;
				$rate_name     = $this->services[ $rate_code ];
				$rate_is_taxed = $this->is_taxed[ $rate_code ];
				$rate_cost     = $cost;

				// Name adjustment.
				if ( ! empty( $this->custom_services[ $rate_code ]['name'] ) ) {
					$rate_name = $this->custom_services[ $rate_code ]['name'];
				}

				// Cost adjustment %.
				if ( ! empty( $this->custom_services[ $rate_code ]['adjustment_percent'] ) ) {
					$rate_cost = $rate_cost + ( $rate_cost * ( floatval( $this->custom_services[ $rate_code ]['adjustment_percent'] ) / 100 ) );
				}

				// Cost adjustment.
				if ( ! empty( $this->custom_services[ $rate_code ]['adjustment'] ) ) {
					$rate_cost = $rate_cost + floatval( $this->custom_services[ $rate_code ]['adjustment'] );
				}

				// Enabled check.
				if ( isset( $this->custom_services[ $rate_code ] ) && empty( $this->custom_services[ $rate_code ]['enabled'] ) ) {
					continue;
				}

				// Sort.
				if ( isset( $this->custom_services[ $rate_code ]['order'] ) ) {
					$sort = $this->custom_services[ $rate_code ]['order'];
				} else {
					$sort = 999;
				}

				$rates[ $rate_id ] = array(
					'id'    => $rate_id,
					'label' => $rate_name,
					'cost'  => $rate_cost,
					'sort'  => $sort,
					'taxes' => $rate_is_taxed,
				);
			}
		}

		// Add rates.
		if ( $rates ) {

			if ( 'all' === $this->offer_rates ) {

				uasort( $rates, array( $this, 'sort_rates' ) );

				foreach ( $rates as $key => $rate ) {
					$this->add_rate( $rate );
				}
			} else {

				$cheapest_rate = '';

				foreach ( $rates as $key => $rate ) {
					if ( ! $cheapest_rate || $cheapest_rate['cost'] > $rate['cost'] ) {
						$cheapest_rate = $rate;
					}
				}

				$cheapest_rate['label'] = $this->title;

				$this->add_rate( $cheapest_rate );
			}
		}
	}

	/**
	 * Sort rates.
	 *
	 * @param mixed $a A.
	 * @param mixed $b B.
	 * @return int
	 */
	public function sort_rates( $a, $b ) {
		if ( $a['sort'] === $b['sort'] ) {
			return 0;
		}
		return ( $a['sort'] < $b['sort'] ) ? -1 : 1;
	}
}
