<?php
/**
 * International Signed rate.
 *
 * @package WC_RoyalMail/Rate
 */

/**
 * RoyalMail_Online_International_Signed class.
 *
 * Updated on 2024-04-02 as per https://www.royalmail.com/sites/royalmail.com/files/2024-03/online-price-guide-april-2024-v1-ta.pdf.
 * See International Signed page 14 - 15.
 */
class RoyalMail_Online_International_Signed extends RoyalMail_Rate {

	/**
	 * ID/Name of rate
	 *
	 * @var string
	 */
	protected $rate_id = 'international_signed';

	const COMPENSATION_UP_TO_VALUE    = 250;
	const COMPENSATION_INCLUDED_VALUE = 50;

	/**
	 * List of countries that support Signed service.
	 *
	 * @see https://www.royalmail.com/business/system/files/Royal-Mail-International-Business-Services-List-March2017.pdf.
	 *
	 * @since 2.5.4
	 * @version 2.5.4
	 *
	 * @var array
	 */
	protected $supported_countries = array(
		'AF',
		'AL',
		'DZ',
		'AO',
		'AI',
		'AG',
		'AM',
		'AW',
		'AU',
		'AZ',
		'BS',
		'BH',
		'BD',
		'BJ',
		'BM',
		'BT',
		'BO',
		'BQ',
		'BA',
		'BW',
		'BR',
		'IO',
		'VG',
		'BN',
		'BF',
		'BI',
		'CM',
		'CV',
		'CF',
		'TD',
		'CL',
		'CN',
		'CX',
		'CO',
		'KM',
		'CG',
		'CD',
		'CR',
		'CU',
		'CW',
		'DJ',
		'DM',
		'DO',
		'EG',
		'SV',
		'GQ',
		'ER',
		'EE',
		'ET',
		'FK',
		'FJ',
		'GF',
		'PF',
		'TF',
		'GA',
		'GM',
		'GH',
		'GD',
		'GP',
		'GT',
		'GN',
		'GW',
		'GY',
		'HT',
		'HN',
		'IN',
		'IR',
		'IQ',
		'IL',
		'CI',
		'JM',
		'JO',
		'KZ',
		'KE',
		'KI',
		'KW',
		'KG',
		'LA',
		'LS',
		'LR',
		'LY',
		'MO',
		'MK',
		'MG',
		'YT',
		'MW',
		'MV',
		'ML',
		'MQ',
		'MR',
		'MU',
		'MX',
		'MN',
		'ME',
		'MS',
		'MA',
		'MZ',
		'MM',
		'NA',
		'NR',
		'NP',
		'NC',
		'NI',
		'NE',
		'NG',
		'NU',
		'KP',
		'NO',
		'OM',
		'PK',
		'PW',
		'PA',
		'PG',
		'PY',
		'PE',
		'PH',
		'PN',
		'PR',
		'QA',
		'RE',
		'RW',
		'ST',
		'SA',
		'SN',
		'SC',
		'SL',
		'SB',
		'ZA',
		'SS',
		'LK',
		'BQ',
		'SH',
		'KN',
		'LC',
		'MF',
		'SX',
		'VC',
		'SD',
		'SR',
		'SZ',
		'SY',
		'TW',
		'TJ',
		'TZ',
		'TL',
		'TG',
		'TK',
		'TN',
		'TM',
		'TC',
		'TV',
		'UA',
		'UY',
		'UZ',
		'VU',
		'VE',
		'VN',
		'WF',
		'EH',
		'WS',
		'YE',
		'ZM',
		'ZW',
	);

	/**
	 * Pricing bands - Europe 1, Europe 2, Europe 3, World Zone 1, World Zone 2.
	 *
	 * @var array
	 */
	protected $bands = array(
		'2024' => array(
			'letter'       => array(
				100 => array( 815, 815, 815, 815, 815 ),
			),
			'large-letter' => array(
				100 => array( 965, 965, 965, 1065, 1080 ),
				250 => array( 1020, 1020, 1020, 1160, 1280 ),
				500 => array( 1100, 1100, 1100, 1360, 1560 ),
				750 => array( 1145, 1145, 1145, 1565, 1865 ),
			),
			'packet'       => array(
				100  => array( 1015, 970, 1095, 1540, 1330 ),
				250  => array( 1015, 970, 1095, 1575, 1365 ),
				500  => array( 1135, 1125, 1250, 1665, 1815 ),
				750  => array( 1240, 1225, 1365, 1905, 2095 ),
				1000 => array( 1335, 1315, 1490, 2170, 2410 ),
				1250 => array( 1390, 1345, 1645, 2370, 2705 ),
				1500 => array( 1400, 1370, 1755, 2550, 3005 ),
				2000 => array( 1415, 1415, 2135, 2655, 3210 ),
			),
		),
	);

	/**
	 * Fixed compensation
	 *
	 * @var string
	 */
	private $compensation = '250';

	/**
	 * Get quotes for this rate
	 *
	 * @param array  $items to be shipped.
	 * @param string $packing_method the method selected.
	 * @param string $destination Address to ship to.
	 * @param array  $boxes User-defined boxes.
	 * @param int    $instance_id Instance ID.
	 *
	 * @return array{ 'international-signed': float }|false|null
	 */
	public function get_quotes( $items, $packing_method, $destination, $boxes = array(), $instance_id = '' ) {
		if ( ! in_array( $destination, $this->supported_countries, true ) ) {
			return array();
		}

		if ( ! empty( $boxes ) ) {
			$this->boxes = array();

			foreach ( $boxes as $key => $box ) {
				$this->boxes[ $key ] = array(
					'length'     => $box['inner_length'],
					'width'      => $box['inner_width'],
					'height'     => $box['inner_height'],
					'box_weight' => $box['box_weight'],
					'weight'     => 2000,
				);
			}
		} else {
			$this->boxes = $this->international_default_box;
		}

		$packages                = $this->get_packages( $items, $packing_method );
		$class_quote             = false;
		$options                 = $this->get_instance_options( $instance_id );
		$ignore_max_compensation = ( ! empty( $options['compensation_optional'] ) && 'yes' === $options['compensation_optional'] );
		$additional_compensation = ( ! empty( $options['enable_addit_compensation'] ) && 'yes' === $options['enable_addit_compensation'] );
		$max_compensation        = ( true === $additional_compensation ) ? self::COMPENSATION_UP_TO_VALUE : self::COMPENSATION_INCLUDED_VALUE;

		if ( $packages ) {
			foreach ( $packages as $package ) {
				if ( $package->value > $max_compensation && ! $ignore_max_compensation ) {
					return false; // Max. compensation is 50 by default but 250 if additional compensation is ticked.
				}

				$quote = $this->get_quote( $package, $destination );

				// Do not return a quote if one of the packages exceeds a limitation.
				if ( false === $quote ) {
					return array();
				}

				$class_quote += $quote;

				if ( $package->value > self::COMPENSATION_INCLUDED_VALUE && $additional_compensation ) {
					$class_quote += $this->compensation;
				}
			}
		}

		// Return pounds.
		$quotes                         = array();
		$quotes['international-signed'] = $class_quote / 100;

		return $quotes;
	}

	/**
	 * Get quote.
	 *
	 * @since 2.5.1
	 * @version 2.5.1
	 *
	 * @param stdClass $package     Package to ship.
	 * @param string   $destination Destination.
	 *
	 * @return bool|int|void
	 */
	public function get_quote( $package, $destination ) {

		$zone = $this->get_zone( $destination );

		$this->validate_package( $package );

		if ( in_array( $package->id, array( 'packet', 'printed-papers' ), true ) && 900 < ( $package->length + $package->width + $package->height ) ) {
			return false; // Exceeding parcels requirement, unpacked.
		}

		if ( ! $this->get_rate_bands( $package->id ) ) {
			return false; // Unpacked item.
		}

		// phpcs:ignore WordPress.PHP.DevelopmentFunctions.error_log_print_r --- Only for debug
		$this->debug( __( 'International signed package:', 'woocommerce-shipping-royalmail' ) . ' <pre>' . print_r( $package, true ) . '</pre>' );

		$bands   = $this->get_rate_bands( $package->id );
		$quote   = 0;
		$matched = false;

		foreach ( $bands as $band => $value ) {
			if ( $package->weight <= $band ) {
				switch ( $zone ) {
					case 'EUR_1':
						$quote += $value[0];
						break;
					case 'EUR_2':
						$quote += $value[1];
						break;
					case 'EUR_3':
					case 'EU':
						$quote += $value[2];
						break;
					case '1':
						$quote += $value[3];
						break;
					case '2':
						$quote += $value[4];
						break;
				}
				$matched = true;
				break;
			}
		}

		if ( ! $matched ) {
			return;
		}
		return $quote;
	}
}
