<?php
/**
 * International Standard rate.
 *
 * @package WC_RoyalMail/Rate
 */

/**
 * RoyalMail_Online_International_Standard class.
 *
 * Updated on 2024-04-02 as per https://www.royalmail.com/sites/royalmail.com/files/2024-03/online-price-guide-april-2024-v1-ta.pdf.
 * See International Standard page 16 - 17.
 */
class RoyalMail_Online_International_Standard extends RoyalMail_Rate {

	/**
	 * ID/Name of rate
	 *
	 * @var string
	 */
	protected $rate_id = 'international_standard';

	const COMPENSATION_UP_TO_VALUE = 20;

	/**
	 * Pricing bands - Europe 1, Europe 2, Europe 3, World Zone 1, World Zone 2, World Zone 3 (previously Zone 1)
	 *
	 * @var array
	 */
	protected $bands = array(
		'2024' => array(
			'letter'       => array(
				100 => array( 250, 250, 250, 250, 250, 250 ),
			),
			'large-letter' => array(
				100 => array( 325, 325, 325, 420, 420, 420 ),
				250 => array( 475, 475, 475, 635, 755, 650 ),
				500 => array( 585, 585, 585, 885, 1090, 920 ),
				750 => array( 695, 695, 695, 1180, 1500, 1230 ),
			),
			'packet'       => array(
				100  => array( 555, 605, 645, 745, 850, 820 ),
				250  => array( 555, 605, 645, 900, 960, 985 ),
				500  => array( 735, 815, 890, 1310, 1480, 1580 ),
				750  => array( 860, 950, 1060, 1615, 1820, 1870 ),
				1000 => array( 990, 1075, 1230, 1915, 2180, 2260 ),
				1250 => array( 1035, 1230, 1360, 2135, 2510, 2615 ),
				1500 => array( 1035, 1230, 1570, 2330, 2850, 2895 ),
				2000 => array( 1200, 1400, 1875, 2450, 3035, 3025 ),
			),
		),
	);

	/**
	 * Get quotes for this rate
	 *
	 * @param array  $items to be shipped.
	 * @param string $packing_method the method selected.
	 * @param string $destination Address to ship to.
	 * @param array  $boxes User-defined boxes.
	 * @param int    $instance_id Instance ID.
	 *
	 * @return array{ 'international-standard': float }|false|null
	 */
	public function get_quotes( $items, $packing_method, $destination, $boxes = array(), $instance_id = '' ) {
		$standard_quote = false;

		if ( ! empty( $boxes ) ) {
			$this->boxes = array();

			foreach ( $boxes as $key => $box ) {
				$this->boxes[ $key ] = array(
					'length'     => $box['inner_length'],
					'width'      => $box['inner_width'],
					'height'     => $box['inner_height'],
					'box_weight' => $box['box_weight'],
					'weight'     => 2000,
				);
			}
		} else {
			$this->boxes = $this->international_default_box;
		}

		$zone                    = $this->get_zone( $destination );
		$options                 = $this->get_instance_options( $instance_id );
		$ignore_max_compensation = ( ! empty( $options['compensation_optional'] ) && 'yes' === $options['compensation_optional'] );
		$packages                = $this->get_packages( $items, $packing_method );

		if ( $packages ) {
			foreach ( $packages as $package ) {
				if ( $package->value > self::COMPENSATION_UP_TO_VALUE && ! $ignore_max_compensation ) {
					return false; // Max. compensation is 20.
				}

				$this->validate_package( $package );

				if ( in_array( $package->id, array( 'packet', 'printed-papers' ), true ) && 900 < ( $package->length + $package->width + $package->height ) ) {
					return false; // Exceeding parcels requirement, unpacked.
				}

				if ( ! $this->get_rate_bands( $package->id ) ) {
					return false; // Unpacked item.
				}

				// phpcs:ignore WordPress.PHP.DevelopmentFunctions.error_log_print_r --- Only for debug
				$this->debug( __( 'International package:', 'woocommerce-shipping-royalmail' ) . ' <pre>' . print_r( $package, true ) . '</pre>' );

				$bands   = $this->get_rate_bands( $package->id );
				$quote   = 0;
				$matched = false;

				foreach ( $bands as $band => $value ) {
					if ( $package->weight <= $band ) {
						switch ( $zone ) {
							case 'EUR_1':
								$quote += $value[0];
								break;
							case 'EUR_2':
								$quote += $value[1];
								break;
							case 'EUR_3':
							case 'EU':
								$quote += $value[2];
								break;
							case '1':
								$quote += $value[3];
								break;
							case '2':
								$quote += $value[4];
								break;
							case '3':
								// Fallback to zone 1 for older prices.
								$quote += isset( $value[5] ) ? $value[5] : $value[3];
								break;
						}
						$matched = true;
						break;
					}
				}

				if ( ! $matched ) {
					return;
				}

				$standard_quote += $quote;
			}
		}

		// Return pounds.
		$quotes                           = array();
		$quotes['international-standard'] = $standard_quote / 100;

		return $quotes;
	}
}
