<?php
/**
 * International-Tracked rate.
 *
 * @package WC_RoyalMail/Rate
 */

/**
 * RoyalMail_Online_International_Tracked class.
 *
 * Updated on 2024-04-02 as per https://www.royalmail.com/sites/royalmail.com/files/2024-03/online-price-guide-april-2024-v1-ta.pdf.
 * See International Tracked page 8 - 9.
 */
class RoyalMail_Online_International_Tracked extends RoyalMail_Rate {

	/**
	 * ID/Name of rate
	 *
	 * @var string
	 */
	protected $rate_id = 'international_tracked';

	const COMPENSATION_UP_TO_VALUE    = 250;
	const COMPENSATION_INCLUDED_VALUE = 50;

	/**
	 * List of countries that support Tracked service.
	 *
	 * @see https://www.royalmail.com/business/system/files/Royal-Mail-International-Business-Services-List-March2017.pdf.
	 *
	 * @since 2.5.4
	 * @version 2.5.4
	 *
	 * @var array
	 */
	protected $supported_countries = array(
		'AX',
		'AD',
		'AU',
		'AT',
		'BE',
		'BR',
		'CA',
		'HR',
		'CY',
		'DK',
		'EE',
		'FO',
		'FI',
		'FR',
		'DE',
		'GI',
		'GR',
		'GL',
		'HK',
		'HU',
		'IS',
		'IN',
		'IE',
		'IL',
		'IT',
		'LV',
		'LB',
		'LI',
		'LT',
		'LU',
		'MY',
		'MT',
		'NL',
		'NZ',
		'NO',
		'PL',
		'PT',
		'RU',
		'SM',
		'RS',
		'SG',
		'SK',
		'SI',
		'KR',
		'ES',
		'SE',
		'CH',
		'TR',
		'US',
		'VA',
	);

	/**
	 * Pricing bands - Ireland, France, Germany, Canada, Europe 1, Europe 2, Europe 3, World Zone 1, World Zone 2, World Zone 3 (previously Zone 1)
	 *
	 * @var array
	 */
	protected $bands = array(
		'2024' => array(
			'letter'         => array(
				100 => array( 790, 790, 790, 790, 790, 790, 790, 790, 790, 790 ),
			),
			'large-letter'   => array(
				100 => array( 950, 950, 950, 1055, 950, 950, 950, 1055, 1060, 1060 ),
				250 => array( 980, 980, 980, 1125, 980, 980, 980, 1125, 1245, 1170 ),
				500 => array( 1090, 1090, 1090, 1350, 1090, 1090, 1090, 1350, 1545, 1380 ),
				750 => array( 1135, 1135, 1135, 1555, 1135, 1135, 1135, 1555, 1855, 1600 ),
			),
			'packet'         => array(
				100  => array( 865, 800, 800, 1375, 865, 900, 985, 1120, 1235, 1170 ),
				250  => array( 865, 800, 800, 1375, 865, 900, 985, 1120, 1235, 1170 ),
				500  => array( 865, 920, 920, 1375, 985, 990, 1170, 1420, 1585, 1490 ),
				750  => array( 1010, 1010, 975, 1620, 1080, 1045, 1305, 1665, 1965, 1550 ),
				1000 => array( 1010, 1075, 995, 1930, 1150, 1110, 1405, 1930, 2310, 1720 ),
				1250 => array( 1010, 1095, 995, 2165, 1190, 1150, 1500, 2175, 2640, 2000 ),
				1500 => array( 1010, 1095, 995, 2335, 1190, 1150, 1680, 2465, 2915, 2000 ),
				2000 => array( 1010, 1095, 995, 2455, 1210, 1215, 1950, 2645, 3110, 2000 ),
			),
			'printed-papers' => array(
				100  => array( 865, 800, 800, 1375, 865, 900, 985, 1120, 1235, 1170 ),
				250  => array( 865, 800, 800, 1375, 865, 900, 985, 1120, 1235, 1170 ),
				500  => array( 865, 920, 920, 1375, 985, 990, 1170, 1420, 1585, 1490 ),
				750  => array( 1010, 1010, 975, 1620, 1080, 1045, 1305, 1665, 1965, 1550 ),
				1000 => array( 1010, 1075, 995, 1930, 1150, 1110, 1405, 1930, 2310, 1720 ),
				1250 => array( 1010, 1095, 995, 2165, 1190, 1150, 1500, 2175, 2640, 2000 ),
				1500 => array( 1010, 1095, 995, 2335, 1190, 1150, 1680, 2465, 2915, 2000 ),
				2000 => array( 1010, 1095, 995, 2455, 1210, 1215, 1950, 2645, 3110, 2000 ),
			),
		),
	);

	/**
	 * Additional rates for printed papers.
	 *
	 * @var Array.
	 */
	protected $additional_rates = array(
		'2024' => array(
			'printed-papers' => array( 130, 130, 130, 180, 130, 130, 130, 180, 225, 250 ),
		),
	);

	/**
	 * Fixed compensation
	 *
	 * @var string
	 */
	private $compensation = '250';

	/**
	 * Get quotes for this rate
	 *
	 * @param array  $items to be shipped.
	 * @param string $packing_method the method selected.
	 * @param string $destination Address to ship to.
	 * @param array  $boxes User-defined boxes.
	 * @param int    $instance_id .
	 *
	 * @return array{ 'international-tracked': float }|false|null
	 */
	public function get_quotes( $items, $packing_method, $destination, $boxes = array(), $instance_id = '' ) {
		if ( ! in_array( $destination, $this->supported_countries, true ) ) {
			return;
		}

		$this->apply_additional_rates();

		$class_quote = false;

		if ( ! empty( $boxes ) ) {
			$this->boxes = array();

			foreach ( $boxes as $key => $box ) {
				$this->boxes[ $key ] = array(
					'length'     => $box['inner_length'],
					'width'      => $box['inner_width'],
					'height'     => $box['inner_height'],
					'box_weight' => $box['box_weight'],
					'weight'     => 2000,
				);
			}
		} else {
			$this->boxes = $this->international_default_box;
		}

		$individual_zones = array( 'IE', 'FR', 'DE', 'CA' );
		$zone             = in_array( $destination, $individual_zones, true ) ? $destination : $this->get_zone( $destination );

		/**
		 * Allow third party to enable/disable printed papers rate.
		 *
		 * @param boolean Flag for enabling/disabling the rate.
		 * @param int Instance ID.
		 * @param string Name of the rate.
		 * @param array Destination.
		 * @param string Packing method.
		 *
		 * @since 3.0.0
		 */
		$printed_paper_packages  = apply_filters( 'woocommerce_shipping_royal_mail_printed_papers_enabled', true, $instance_id, 'tracked', $destination, $packing_method ) ? $this->get_printed_papers_packages( $items, $destination, $packing_method ) : array();
		$regular_packages        = $this->get_packages( $items, $packing_method );
		$packages                = array_merge( $regular_packages, $printed_paper_packages );
		$options                 = $this->get_instance_options( $instance_id );
		$ignore_max_compensation = ( ! empty( $options['compensation_optional'] ) && 'yes' === $options['compensation_optional'] );
		$additional_compensation = ( ! empty( $options['enable_addit_compensation'] ) && 'yes' === $options['enable_addit_compensation'] );
		$max_compensation        = ( true === $additional_compensation ) ? self::COMPENSATION_UP_TO_VALUE : self::COMPENSATION_INCLUDED_VALUE;

		if ( $packages ) {
			foreach ( $packages as $package ) {

				$this->validate_package( $package );

				if ( in_array( $package->id, array( 'packet', 'printed-papers' ), true ) && 900 < ( $package->length + $package->width + $package->height ) ) {
					return false; // Exceeding parcels requirement, unpacked.
				}

				if ( $package->value > $max_compensation && ! $ignore_max_compensation ) {
					return false;
				}

				if ( ! $this->get_rate_bands( $package->id ) ) {
					return false; // Unpacked item.
				}

				// phpcs:ignore WordPress.PHP.DevelopmentFunctions.error_log_print_r --- Only for debug
				$this->debug( __( 'International tracked package:', 'woocommerce-shipping-royalmail' ) . ' <pre>' . print_r( $package, true ) . '</pre>' );

				$bands   = $this->get_rate_bands( $package->id );
				$quote   = 0;
				$matched = false;

				foreach ( $bands as $band => $value ) {
					if ( $package->weight <= $band ) {
						switch ( $zone ) {
							case 'IE':
								$quote += $value[0];
								break;
							case 'FR':
								$quote += $value[1];
								break;
							case 'DE':
								$quote += $value[2];
								break;
							case 'CA':
								$quote += $value[3];
								break;
							case 'EUR_1':
								$quote += $value[4];
								break;
							case 'EUR_2':
								$quote += $value[5];
								break;
							case 'EUR_3':
							case 'EU':
								$quote += $value[6];
								break;
							case '1':
								$quote += $value[7];
								break;
							case '2':
								$quote += $value[8];
								break;
							case '3':
								// Fallback to zone 1 for older prices.
								$quote += isset( $value[9] ) ? $value[9] : $value[7];
								break;
						}
						$matched = true;
						break;
					}
				}

				if ( ! $matched ) {
					return;
				}

				$class_quote += $quote;

				if ( $package->value > self::COMPENSATION_INCLUDED_VALUE && $additional_compensation ) {
					$class_quote += $this->compensation;
				}
			}
		}

		// Return pounds.
		$quotes                          = array();
		$quotes['international-tracked'] = $class_quote / 100;

		return $quotes;
	}
}
