<?php
/**
 * International Standard rate.
 *
 * @package WC_RoyalMail/Rate
 */

/**
 * RoyalMail_Regular_International_Standard class.
 *
 * Updated on 2024-04-02 as per https://www.royalmail.com/sites/royalmail.com/files/2024-03/our-prices-april-2024-v1-ta.pdf.
 * See International Standard page 17.
 */
class RoyalMail_Regular_International_Standard extends RoyalMail_Rate {

	/**
	 * ID/Name of rate
	 *
	 * @var string
	 */
	protected $rate_id = 'international_standard';

	const COMPENSATION_UP_TO_VALUE = 20;

	/**
	 * Pricing bands - Europe, Zone 1, Zone 2, Zone 3 (previously Zone 1)
	 *
	 * @var array
	 */
	protected $bands = array(
		'2024' => array(
			'letter'         => array(
				100 => array( 250, 250, 250, 250, 250, 250 ),
			),
			'large-letter'   => array(
				100 => array( 325, 325, 325, 420, 420, 420 ),
				250 => array( 545, 545, 545, 705, 825, 720 ),
				500 => array( 655, 655, 655, 955, 1160, 990 ),
				750 => array( 765, 765, 765, 1250, 1570, 1300 ),
			),
			'packet'         => array(
				100  => array( 830, 845, 925, 1100, 1245, 1250 ),
				250  => array( 830, 845, 925, 1250, 1350, 1425 ),
				500  => array( 1040, 1075, 1140, 1700, 1870, 2045 ),
				750  => array( 1170, 1210, 1295, 2000, 2210, 2350 ),
				1000 => array( 1305, 1335, 1445, 2310, 2570, 2750 ),
				1250 => array( 1415, 1470, 1575, 2575, 2900, 3130 ),
				1500 => array( 1415, 1470, 1700, 2820, 3240, 3430 ),
				2000 => array( 1580, 1635, 1850, 2955, 3425, 3570 ),
			),
			'printed-papers' => array(
				100  => array( 830, 845, 925, 1100, 1245, 1250 ),
				250  => array( 830, 845, 925, 1250, 1350, 1425 ),
				500  => array( 1040, 1075, 1140, 1700, 1870, 2045 ),
				750  => array( 1170, 1210, 1295, 2000, 2210, 2350 ),
				1000 => array( 1305, 1335, 1445, 2310, 2570, 2750 ),
				1250 => array( 1415, 1470, 1575, 2575, 2900, 3130 ),
				1500 => array( 1415, 1470, 1700, 2820, 3240, 3430 ),
				2000 => array( 1580, 1635, 1850, 2955, 3425, 3570 ),
			),
		),
	);

	/**
	 * Additional rates for printed papers.
	 *
	 * @var Array.
	 */
	protected $additional_rates = array(
		'2024' => array(
			'printed-papers' => array( 155, 155, 155, 210, 260, 285 ),
		),
	);

	/**
	 * Get quotes for this rate
	 *
	 * @param array  $items to be shipped.
	 * @param string $packing_method the method selected.
	 * @param string $destination Address to ship to.
	 * @param array  $boxes User-defined boxes.
	 * @param int    $instance_id Instance ID.
	 *
	 * @return array{ 'international-standard': float }|false|null
	 */
	public function get_quotes( $items, $packing_method, $destination, $boxes = array(), $instance_id = '' ) {
		$standard_quote = false;

		$this->apply_additional_rates();

		if ( ! empty( $boxes ) ) {
			$this->boxes = array();

			foreach ( $boxes as $key => $box ) {
				$this->boxes[ $key ] = array(
					'length'     => $box['inner_length'],
					'width'      => $box['inner_width'],
					'height'     => $box['inner_height'],
					'box_weight' => $box['box_weight'],
					'weight'     => 2000,
				);
			}
		} else {
			$this->boxes = $this->international_default_box;
		}

		$zone                    = $this->get_zone( $destination );
		$options                 = $this->get_instance_options( $instance_id );
		$ignore_max_compensation = ( ! empty( $options['compensation_optional'] ) && 'yes' === $options['compensation_optional'] );

		/**
		 * Allow third party to enable/disable printed papers rate.
		 *
		 * @param boolean Flag for enabling/disabling the rate.
		 * @param int Instance ID.
		 * @param string Name of the rate.
		 * @param array Destination.
		 * @param string Packing method.
		 *
		 * @since 3.0.0
		 */
		$printed_paper_packages = apply_filters( 'woocommerce_shipping_royal_mail_printed_papers_enabled', true, $instance_id, 'standard', $destination, $packing_method ) ? $this->get_printed_papers_packages( $items, $destination, $packing_method ) : array();
		$regular_packages       = $this->get_packages( $items, $packing_method );
		$packages               = array_merge( $regular_packages, $printed_paper_packages );

		if ( $packages ) {
			foreach ( $packages as $package ) {
				if ( $package->value > self::COMPENSATION_UP_TO_VALUE && ! $ignore_max_compensation ) {
					return false; // Max. compensation is 20.
				}

				$this->validate_package( $package );

				if ( in_array( $package->id, array( 'packet', 'printed-papers' ), true ) && 900 < ( $package->length + $package->width + $package->height ) ) {
					return false; // Exceeding parcels requirement, unpacked.
				}

				if ( ! $this->get_rate_bands( $package->id ) ) {
					return false; // Unpacked item.
				}

				// phpcs:ignore WordPress.PHP.DevelopmentFunctions.error_log_print_r --- Only for debug
				$this->debug( __( 'International package:', 'woocommerce-shipping-royalmail' ) . ' <pre>' . print_r( $package, true ) . '</pre>' );

				$bands   = $this->get_rate_bands( $package->id );
				$quote   = 0;
				$matched = false;

				foreach ( $bands as $band => $value ) {
					if ( $package->weight <= $band ) {
						switch ( $zone ) {
							case 'EUR_1':
								$quote += $value[0];
								break;
							case 'EUR_2':
								$quote += $value[1];
								break;
							case 'EUR_3':
							case 'EU':
								$quote += $value[2];
								break;
							case '1':
								$quote += $value[3];
								break;
							case '2':
								$quote += $value[4];
								break;
							case '3':
								// Fallback to zone 1 for older prices.
								$quote += isset( $value[5] ) ? $value[5] : $value[3];
								break;
						}
						$matched = true;
						break;
					}
				}

				if ( ! $matched ) {
					return;
				}

				$standard_quote += $quote;
			}
		}

		// Return pounds.
		$quotes                           = array();
		$quotes['international-standard'] = $standard_quote / 100;

		return $quotes;
	}
}
