<?php
/**
 * International-Tracked-and-Signed rate.
 *
 * @package WC_RoyalMail/Rate
 */

/**
 * RoyalMail_Regular_International_Tracked_Signed class.
 *
 * Updated on 2024-04-02 as per https://www.royalmail.com/sites/royalmail.com/files/2024-03/our-prices-april-2024-v1-ta.pdf.
 * See International Tracked & Signed page 13 - 14.
 */
class RoyalMail_Regular_International_Tracked_Signed extends RoyalMail_Rate {

	/**
	 * ID/Name of rate
	 *
	 * @var string
	 */
	protected $rate_id = 'international_tracked_signed';

	const COMPENSATION_UP_TO_VALUE    = 250;
	const COMPENSATION_INCLUDED_VALUE = 50;

	/**
	 * List of countries that support Tracked and Signed service.
	 *
	 * @see https://www.royalmail.com/business/system/files/Royal-Mail-International-Business-Services-List-March2017.pdf
	 *
	 * @since 2.5.4
	 * @version 2.5.4
	 *
	 * @var array
	 */
	protected $supported_countries = array(
		'AX',
		'AD',
		'AR',
		'AT',
		'BB',
		'BY',
		'BE',
		'BZ',
		'BG',
		'KH',
		'CA',
		'KY',
		'CK',
		'HR',
		'CY',
		'CZ',
		'DK',
		'EC',
		'FO',
		'FI',
		'FR',
		'GE',
		'DE',
		'GI',
		'GR',
		'GL',
		'HK',
		'HU',
		'IS',
		'ID',
		'IE',
		'IT',
		'JP',
		'LV',
		'LB',
		'LI',
		'LT',
		'LU',
		'MY',
		'MT',
		'MD',
		'NL',
		'NZ',
		'PL',
		'PT',
		'RO',
		'RU',
		'SM',
		'RS',
		'SG',
		'SK',
		'SI',
		'KR',
		'ES',
		'SE',
		'CH',
		'TH',
		'TO',
		'TT',
		'TR',
		'UG',
		'AE',
		'US',
		'VA',
	);

	/**
	 * Pricing bands - Europe, Zone 1, Zone 2, Zone 3 (previously Zone 1)
	 *
	 * @var array
	 */
	protected $bands = array(
		'2024' => array(
			'letter'         => array(
				100 => array( 815, 815, 815, 815, 815, 815 ),
			),
			'large-letter'   => array(
				100 => array( 965, 965, 965, 1065, 1080, 1075 ),
				250 => array( 1090, 1090, 1090, 1230, 1350, 1250 ),
				500 => array( 1170, 1170, 1170, 1430, 1630, 1465 ),
				750 => array( 1215, 1215, 1215, 1635, 1935, 1685 ),
			),
			'packet'         => array(
				100  => array( 1375, 1390, 1530, 1770, 1900, 1845 ),
				250  => array( 1375, 1390, 1530, 1810, 1935, 2020 ),
				500  => array( 1520, 1560, 1700, 2240, 2430, 2615 ),
				750  => array( 1635, 1670, 1825, 2500, 2740, 2835 ),
				1000 => array( 1740, 1770, 1955, 2795, 3085, 3235 ),
				1250 => array( 1800, 1805, 2025, 3015, 3355, 3615 ),
				1500 => array( 1810, 1830, 2090, 3170, 3630, 3915 ),
				2000 => array( 1825, 1880, 2140, 3220, 3745, 3970 ),
			),
			'printed-papers' => array(
				100  => array( 1375, 1390, 1530, 1770, 1900, 1845 ),
				250  => array( 1375, 1390, 1530, 1810, 1935, 2020 ),
				500  => array( 1520, 1560, 1700, 2240, 2430, 2615 ),
				750  => array( 1635, 1670, 1825, 2500, 2740, 2835 ),
				1000 => array( 1740, 1770, 1955, 2795, 3085, 3235 ),
				1250 => array( 1800, 1805, 2025, 3015, 3355, 3615 ),
				1500 => array( 1810, 1830, 2090, 3170, 3630, 3915 ),
				2000 => array( 1825, 1880, 2140, 3220, 3745, 3970 ),
			),
		),
	);

	/**
	 * Additional rates for printed papers.
	 *
	 * @var Array.
	 */
	protected $additional_rates = array(
		'2024' => array(
			'printed-papers' => array( 155, 155, 155, 210, 260, 285 ),
		),
	);

	/**
	 * Fixed compensation
	 *
	 * @var string
	 */
	private $compensation = '250';

	/**
	 * Get quotes for this rate.
	 *
	 * @since 2.5.4
	 * @version 2.5.4
	 *
	 * @param array  $items to be shipped.
	 * @param string $packing_method the method selected.
	 * @param string $destination Address to ship to.
	 * @param array  $boxes User-defined boxes.
	 * @param int    $instance_id Instance ID.
	 *
	 * @return array{ 'international-tracked-signed': float }|false|null
	 */
	public function get_quotes( $items, $packing_method, $destination, $boxes = array(), $instance_id = '' ) {
		if ( ! in_array( $destination, $this->supported_countries, true ) ) {
			return array();
		}

		$this->apply_additional_rates();

		$class_quote = false;

		if ( ! empty( $boxes ) ) {
			$this->boxes = array();

			foreach ( $boxes as $key => $box ) {
				$this->boxes[ $key ] = array(
					'length'     => $box['inner_length'],
					'width'      => $box['inner_width'],
					'height'     => $box['inner_height'],
					'box_weight' => $box['box_weight'],
					'weight'     => 2000,
				);
			}
		} else {
			$this->boxes = $this->international_default_box;
		}

		$zone = $this->get_zone( $destination );

		/**
		 * Allow third party to enable/disable printed papers rate.
		 *
		 * @param boolean Flag for enabling/disabling the rate.
		 * @param int Instance ID.
		 * @param string Name of the rate.
		 * @param array Destination.
		 * @param string Packing method.
		 *
		 * @since 3.0.0
		 */
		$printed_paper_packages  = apply_filters( 'woocommerce_shipping_royal_mail_printed_papers_enabled', true, $instance_id, 'tracked-signed', $destination, $packing_method ) ? $this->get_printed_papers_packages( $items, $destination, $packing_method ) : array();
		$regular_packages        = $this->get_packages( $items, $packing_method );
		$packages                = array_merge( $regular_packages, $printed_paper_packages );
		$options                 = $this->get_instance_options( $instance_id );
		$ignore_max_compensation = ( ! empty( $options['compensation_optional'] ) && 'yes' === $options['compensation_optional'] );
		$additional_compensation = ( ! empty( $options['enable_addit_compensation'] ) && 'yes' === $options['enable_addit_compensation'] );
		$max_compensation        = ( true === $additional_compensation ) ? self::COMPENSATION_UP_TO_VALUE : self::COMPENSATION_INCLUDED_VALUE;

		if ( $packages ) {
			foreach ( $packages as $package ) {

				$this->validate_package( $package );

				if ( in_array( $package->id, array( 'packet', 'printed-papers' ), true ) && 900 < ( $package->length + $package->width + $package->height ) ) {
					return false; // Exceeding parcels requirement, unpacked.
				}

				if ( $package->value > $max_compensation && ! $ignore_max_compensation ) {
					return false;
				}

				if ( ! $this->get_rate_bands( $package->id ) ) {
					return false; // Unpacked item.
				}

				// phpcs:ignore WordPress.PHP.DevelopmentFunctions.error_log_print_r --- Only for debug
				$this->debug( __( 'International tracked and signed package:', 'woocommerce-shipping-royalmail' ) . ' <pre>' . print_r( $package, true ) . '</pre>' );

				$bands   = $this->get_rate_bands( $package->id );
				$quote   = 0;
				$matched = false;

				foreach ( $bands as $band => $value ) {
					if ( $package->weight <= $band ) {
						switch ( $zone ) {
							case 'EUR_1':
								$quote += $value[0];
								break;
							case 'EUR_2':
								$quote += $value[1];
								break;
							case 'EUR_3':
							case 'EU':
								$quote += $value[2];
								break;
							case '1':
								$quote += $value[3];
								break;
							case '2':
								$quote += $value[4];
								break;
							case '3':
								// Fallback to zone 1 for older prices.
								$quote += isset( $value[5] ) ? $value[5] : $value[3];
								break;
						}
						$matched = true;
						break;
					}
				}

				if ( ! $matched ) {
					return;
				}

				$class_quote += $quote;

				if ( $package->value > self::COMPENSATION_INCLUDED_VALUE && $additional_compensation ) {
					$class_quote += $this->compensation;
				}
			}
		}

		// Return pounds.
		$quotes                                 = array();
		$quotes['international-tracked-signed'] = $class_quote / 100;

		return $quotes;
	}
}
