<?php
/**
 * International-Tracked rate.
 *
 * @package WC_RoyalMail/Rate
 */

/**
 * RoyalMail_Regular_International_Tracked class.
 *
 * Updated on 2024-04-02 as per https://www.royalmail.com/sites/royalmail.com/files/2024-03/our-prices-april-2024-v1-ta.pdf.
 * See International Tracked page 11 - 12.
 */
class RoyalMail_Regular_International_Tracked extends RoyalMail_Rate {

	/**
	 * ID/Name of rate
	 *
	 * @var string
	 */
	protected $rate_id = 'international_tracked';

	const COMPENSATION_UP_TO_VALUE    = 250;
	const COMPENSATION_INCLUDED_VALUE = 50;

	/**
	 * List of countries that support Tracked service.
	 *
	 * @see https://www.royalmail.com/business/system/files/Royal-Mail-International-Business-Services-List-March2017.pdf
	 *
	 * @since 2.5.4
	 * @version 2.5.4
	 *
	 * @var array
	 */
	protected $supported_countries = array(
		'AX',
		'AD',
		'AU',
		'AT',
		'BE',
		'BR',
		'CA',
		'HR',
		'CY',
		'DK',
		'EE',
		'FO',
		'FI',
		'FR',
		'DE',
		'GI',
		'GR',
		'GL',
		'HK',
		'HU',
		'IS',
		'IN',
		'IE',
		'IL',
		'IT',
		'LV',
		'LB',
		'LI',
		'LT',
		'LU',
		'MY',
		'MT',
		'NL',
		'NZ',
		'NO',
		'PL',
		'PT',
		'RU',
		'SM',
		'RS',
		'SG',
		'SK',
		'SI',
		'KR',
		'ES',
		'SE',
		'CH',
		'TR',
		'US',
		'VA',
	);

	/**
	 * Pricing bands - Europe, Zone 1, Zone 2, Zone 3 (previously Zone 1)
	 *
	 * @var array
	 */
	protected $bands = array(
		'2024' => array(
			'letter'         => array(
				100 => array( 790, 790, 790, 790, 790, 790 ),
			),
			'large-letter'   => array(
				100 => array( 950, 950, 950, 1055, 1060, 1060 ),
				250 => array( 1050, 1050, 1050, 1215, 1335, 1240 ),
				500 => array( 1160, 1160, 1160, 1420, 1615, 1450 ),
				750 => array( 1205, 1205, 1205, 1625, 1925, 1670 ),
			),
			'packet'         => array(
				250  => array( 1205, 1240, 1325, 1525, 1670, 1720 ),
				500  => array( 1335, 1370, 1500, 1970, 2180, 2185 ),
				750  => array( 1435, 1475, 1595, 2240, 2485, 2470 ),
				1000 => array( 1500, 1545, 1670, 2530, 2810, 2765 ),
				1250 => array( 1550, 1590, 1755, 2745, 3120, 3185 ),
				1500 => array( 1550, 1590, 1845, 2900, 3395, 3185 ),
				2000 => array( 1550, 1745, 1920, 3020, 3580, 3185 ),
			),
			'printed-papers' => array(
				250  => array( 1205, 1240, 1325, 1525, 1670, 1720 ),
				500  => array( 1335, 1370, 1500, 1970, 2180, 2185 ),
				750  => array( 1435, 1475, 1595, 2240, 2485, 2470 ),
				1000 => array( 1500, 1545, 1670, 2530, 2810, 2765 ),
				1250 => array( 1550, 1590, 1755, 2745, 3120, 3185 ),
				1500 => array( 1550, 1590, 1845, 2900, 3395, 3185 ),
				2000 => array( 1550, 1745, 1920, 3020, 3580, 3185 ),
			),
		),
	);

	/**
	 * Additional rates for printed papers.
	 *
	 * @var Array.
	 */
	protected $additional_rates = array(
		'2024' => array(
			'printed-papers' => array( 155, 155, 155, 210, 260, 285 ),
		),
	);

	/**
	 * Fixed compensation
	 *
	 * @var string
	 */
	private $compensation = '250';

	/**
	 * Get quotes for this rate
	 *
	 * @param array  $items to be shipped.
	 * @param string $packing_method the method selected.
	 * @param string $destination Address to ship to.
	 * @param array  $boxes User-defined boxes.
	 * @param int    $instance_id Instance ID.
	 *
	 * @return array{ 'international-tracked': float }|false|null
	 */
	public function get_quotes( $items, $packing_method, $destination, $boxes = array(), $instance_id = '' ) {
		if ( ! in_array( $destination, $this->supported_countries, true ) ) {
			return;
		}

		$this->apply_additional_rates();

		$class_quote = false;

		if ( ! empty( $boxes ) ) {
			$this->boxes = array();

			foreach ( $boxes as $key => $box ) {
				$this->boxes[ $key ] = array(
					'length'     => $box['inner_length'],
					'width'      => $box['inner_width'],
					'height'     => $box['inner_height'],
					'box_weight' => $box['box_weight'],
					'weight'     => 2000,
				);
			}
		} else {
			$this->boxes = $this->international_default_box;
		}

		$zone = $this->get_zone( $destination );

		/**
		 * Allow third party to enable/disable printed papers rate.
		 *
		 * @param boolean Flag for enabling/disabling the rate.
		 * @param int Instance ID.
		 * @param string Name of the rate.
		 * @param array Destination.
		 * @param string Packing method.
		 *
		 * @since 3.0.0
		 */
		$printed_paper_packages  = apply_filters( 'woocommerce_shipping_royal_mail_printed_papers_enabled', true, $instance_id, 'tracked', $destination, $packing_method ) ? $this->get_printed_papers_packages( $items, $destination, $packing_method ) : array();
		$regular_packages        = $this->get_packages( $items, $packing_method );
		$packages                = array_merge( $regular_packages, $printed_paper_packages );
		$options                 = $this->get_instance_options( $instance_id );
		$ignore_max_compensation = ( ! empty( $options['compensation_optional'] ) && 'yes' === $options['compensation_optional'] );
		$additional_compensation = ( ! empty( $options['enable_addit_compensation'] ) && 'yes' === $options['enable_addit_compensation'] );
		$max_compensation        = ( true === $additional_compensation ) ? self::COMPENSATION_UP_TO_VALUE : self::COMPENSATION_INCLUDED_VALUE;

		if ( $packages ) {
			foreach ( $packages as $package ) {

				$this->validate_package( $package );

				if ( in_array( $package->id, array( 'packet', 'printed-papers' ), true ) && 900 < ( $package->length + $package->width + $package->height ) ) {
					return false; // Exceeding parcels requirement, unpacked.
				}

				if ( $package->value > $max_compensation && ! $ignore_max_compensation ) {
					return false;
				}

				if ( ! $this->get_rate_bands( $package->id ) ) {
					return false; // Unpacked item.
				}

				// phpcs:ignore WordPress.PHP.DevelopmentFunctions.error_log_print_r --- Only for debug
				$this->debug( __( 'International tracked package:', 'woocommerce-shipping-royalmail' ) . ' <pre>' . print_r( $package, true ) . '</pre>' );

				$bands   = $this->get_rate_bands( $package->id );
				$quote   = 0;
				$matched = false;

				foreach ( $bands as $band => $value ) {
					if ( $package->weight <= $band ) {
						switch ( $zone ) {
							case 'EUR_1':
								$quote += $value[0];
								break;
							case 'EUR_2':
								$quote += $value[1];
								break;
							case 'EUR_3':
							case 'EU':
								$quote += $value[2];
								break;
							case '1':
								$quote += $value[3];
								break;
							case '2':
								$quote += $value[4];
								break;
							case '3':
								// Fallback to zone 1 for older prices.
								$quote += isset( $value[5] ) ? $value[5] : $value[3];
								break;
						}
						$matched = true;
						break;
					}
				}

				if ( ! $matched ) {
					return;
				}

				$class_quote += $quote;

				if ( $package->value > self::COMPENSATION_INCLUDED_VALUE && $additional_compensation ) {
					$class_quote += $this->compensation;
				}
			}
		}

		// Return pounds.
		$quotes                          = array();
		$quotes['international-tracked'] = $class_quote / 100;

		return $quotes;
	}
}
