<?php
/**
 * First class rate.
 *
 * @package WC_RoyalMail/Rate
 */

/**
 * RoyalMail_Online_Tracked_48 class.
 *
 * Updated on 2024-04-02 as per https://www.royalmail.com/sites/royalmail.com/files/2024-03/online-price-guide-april-2024-v1-ta.pdf.
 * See UK Tracked page 5.
 */
class RoyalMail_Online_Tracked_48 extends RoyalMail_Rate {

	/**
	 * ID/Name of rate.
	 *
	 * @var string
	 */
	protected $rate_id = 'tracked_48';

	const COMPENSATION_UP_TO_VALUE = 150;

	/**
	 * Pricing bands
	 *
	 * Key is size (e.g. 'letter') and value is an array where key is weight in
	 * gram and value is the price (in penny).
	 *
	 * @var array
	 */
	protected $bands = array(
		'2024' => array(
			'large-letter'        => array(
				750 => 270,
			),
			'small-parcel-wide'   => array(
				2000 => 339,
			),
			'small-parcel-deep'   => array(
				2000 => 339,
			),
			'small-parcel-bigger' => array(
				2000 => 339,
			),
			'medium-parcel'       => array(
				2000  => 509,
				10000 => 659,
				20000 => 1019,
			),
			'tube'                => array(
				2000  => 509,
				10000 => 659,
				20000 => 1019,
			),
		),
	);

	/**
	 * Shipping boxes.
	 *
	 * @var array
	 */
	protected $boxes = array(
		'large-letter'        => array(
			'length' => 353,
			'width'  => 250,
			'height' => 25,
			'weight' => 750,
		),
		'small-parcel-wide'   => array(
			'length' => 450,
			'width'  => 350,
			'height' => 80,
			'weight' => 2000,
		),
		'small-parcel-deep'   => array(
			'length' => 350,
			'width'  => 250,
			'height' => 160,
			'weight' => 2000,
		),
		'small-parcel-bigger' => array(
			'length' => 450,
			'width'  => 350,
			'height' => 160,
			'weight' => 2000,
		),
		'medium-parcel'       => array(
			'length' => 610,
			'width'  => 460,
			'height' => 460,
			'weight' => 20000,
		),
		'tube'                => array(
			'length' => 900,
			'width'  => 70,
			'height' => 70,
			'weight' => 2000,
		),
	);

	/**
	 * Get quotes for this rate.
	 *
	 * @param  array  $items to be shipped.
	 * @param  string $packing_method the method selected.
	 * @param  string $destination Address to ship to.
	 * @param  array  $boxes Boxes used.
	 * @param  int    $instance_id Instance ID.
	 * @return array{ 'tracked-48': float }|false|null
	 */
	public function get_quotes( $items, $packing_method, $destination, $boxes = array(), $instance_id = '' ) {
		$class_quote = 0;

		/**
		 * Allow third party to enable/disable tube rate.
		 *
		 * @param boolean Flag for enabling/disabling the rate.
		 * @param int Instance ID.
		 * @param string Name of the rate.
		 * @param array Destination.
		 * @param string Packing method.
		 *
		 * @since 3.2.4
		 */
		$tube_packages           = apply_filters( 'woocommerce_shipping_royal_mail_tube_enabled', true, $instance_id, 'tracked-24', $destination, $packing_method ) ? $this->get_tube_packages( $items, $destination, $packing_method ) : array();
		$regular_packages        = $this->get_packages( $items, $packing_method );
		$packages                = array_merge( $regular_packages, $tube_packages );
		$options                 = $this->get_instance_options( $instance_id );
		$ignore_max_compensation = ( ! empty( $options['compensation_optional'] ) && 'yes' === $options['compensation_optional'] );

		if ( $packages ) {
			foreach ( $packages as $package ) {
				if ( $package->value > self::COMPENSATION_UP_TO_VALUE && ! $ignore_max_compensation ) {
					return false; // Max. compensation is 150.
				}

				$quote = 0;

				if ( ! $this->get_rate_bands( $package->id ) ) {
					return false; // Unpacked item.
				}

				// phpcs:ignore WordPress.PHP.DevelopmentFunctions.error_log_print_r --- Only for debug
				$this->debug( __( 'RoyalMail Tracked 48:', 'woocommerce-shipping-royalmail' ) . ' <pre>' . print_r( $package, true ) . '</pre>' );

				$bands = $this->get_rate_bands( $package->id );

				$matched = false;

				foreach ( $bands as $band => $value ) {
					if ( is_numeric( $band ) && $package->weight <= $band ) {
						$quote  += $value;
						$matched = true;
						break;
					}
				}

				if ( ! $matched ) {
					return null;
				}

				$class_quote += $quote;
			}
		}

		// Return pounds.
		$quotes               = array();
		$quotes['tracked-48'] = $class_quote / 100;

		return $quotes;
	}
}
